%gross measurement error analysis (GMEA)
%use hypothesis testing to detect gross error in measurement set
%author Lake-Ee Quek, The University of Queensland, AIBN
%08-March-2009
clear all;clc;
%read SBML model, select either CHO or hybridoma model
[stoicMat, rxnTable,speciesList] = sbmlREAD('hybridomaMFA_minimum.xml');%hybridoma model
% [stoicMat, rxnTable,speciesList] = sbmlREAD('CHOMFA_minimum.xml');%CHO model


%select reaction to delete
rxn_del ={'EF0031'};%hypoxanthine uptake
% rxn_del=[];
for i = 1:length(rxn_del)
    rxnID = rxn_del{i};
    for j = 1:size(rxnTable,1)
        rxnString = rxnTable{j,1};
        if strcmp(rxnString,rxnID)
            rxnTable(j,:)=[];
            stoicMat(j,:)=[];
            break
        end
    end
end

%read input data
%column 1: rxnID (string);%column 2: rates (double)
%column 3: standard error (double)
disp('input file must have 3 columns: rxnID, rates, standard error')
ratesFile = input('specify name of file containing measured rates  >','s');
[rxnMea,rates,se] = textread(ratesFile,'%s%f%f');

%separate stoic matrix (Sm'vm + Sc'vc = 0)
%into measured and calculated
rowMea=[];
for i = 1:length(rxnMea)
    rxnMeaID = rxnMea{i};
    for j = 1:size(rxnTable,1)
        rxnID = rxnTable{j,1};
        if strcmp(rxnMeaID,rxnID)
            rowMea(i)= j;
        end
    end
end
rowCalc=[];
for i = 1:size(stoicMat,1)
    if length(find(rowMea==i)) == 0
        rowCalc=[rowCalc i];
    end
end
stoicMea = stoicMat(rowMea,:);
stoicCalc = stoicMat;
stoicCalc(rowMea,:)=[];

%calculate the redundancy matrix
rm = stoicMea' - stoicCalc'* pinv(stoicCalc')* stoicMea';
%rank shows degree of redundancy
disp(['rank:   ' num2str(rank(rm,1e-3))]);

%rank should be 2 or greater, pick n rank rows for metabolite balance
%find the combination of rows that give the smallest conditional number
%generate combination
count = rank(rm,1e-3);
maxCount = size(rm,1);
combStore=[];
firstComb = [1:count];
lastComb =[maxCount-count+1:maxCount];
combStore=[combStore;firstComb];
while 1
    latestComb = combStore(end,:);
    updatedCell = 0;
    for cellCheck = count:-1:1
        if latestComb(cellCheck) < lastComb(cellCheck)
            latestComb(cellCheck) = latestComb(cellCheck) + 1;
            updatedCell = cellCheck;
            break;
        end
    end
    if updatedCell > 0
        increment = 1;
        for cellMod = updatedCell+1:count
            latestComb(cellMod) = latestComb(updatedCell)+increment;
            increment = increment + 1;
        end
        combStore=[combStore;latestComb];
    end
    if cellCheck == 1 && updatedCell == 0
        break
    end
end

%screen combination
rowSelect =combStore(1,:);
condStore=cond(rm(rowSelect,:));
for i = 1:size(combStore,1)
    if cond(rm(combStore(i,:),:)) < condStore
        condStore = cond(rm(combStore(i,:),:));
        rowSelect=combStore(i,:);
    end
end

%generate reduced redundancy matrix
rm_reduced = rm(rowSelect,:);
%calculate residual error
res_error = rm_reduced*rates;
%generate variance-covariance matrix
%assume independent measurements, error occupies diagonal elements
F=diag(se.^2); 
P=rm_reduced*F*rm_reduced';

%test hypothesis, DoF 2, % s chi value of    5.99
h_test = res_error'*inv(P)*res_error;
fprintf('test score:\t%1.3f\n',h_test);
fprintf('chi score cut-off:\t%1.3f\n',chi2inv(0.95,count));

%minimize error and adjust measured rates
del_hat = F*rm_reduced'*inv(P)*rm_reduced*rates;
rates_hat = rates - del_hat;

%calculate variance of adjusted rates
F_hat = F - F*rm_reduced'*inv(P)*rm_reduced*F;

%calculate unknown rates and associated variance
rates_calc_hat = - pinv(stoicCalc')*stoicMea'*rates_hat;
F_calc_hat = pinv(stoicCalc')*stoicMea'*F_hat*stoicMea*pinv(stoicCalc')';

dataIN = [rates_hat F_hat];